package com.yfd.platform.modules.experimentalData.controller;


import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yfd.platform.annotation.Log;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.experimentalData.domain.TsTask;
import com.yfd.platform.modules.experimentalData.service.ITsTaskService;
import com.yfd.platform.modules.specialDocument.domain.Project;
import com.yfd.platform.utils.StringUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 试验任务表 前端控制器
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-21
 */
@RestController
@RequestMapping("/experimentalData/tstask")
public class TsTaskController {

    //试验任务服务类
    @Resource
    private ITsTaskService tsTaskService;

    /**********************************
     * 用途说明: 分页查询试验数据管理-试验任务管理
     * 参数说明
     * taskName      任务名称
     * startDate     (开始日期)
     * endDate      （结束日期）
     * taskPlace     任务地点
     * taskPerson    任务人员
     * taskCode      任务编号
     * taskType      任务类型
     * carrierName    任务人员
     * deviceCode    任务人员
     * testDescribe    任务人员
     * sensorDescribe    任务人员
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Log(module = "试验数据管理", value = "分页查询试验数据管理试验任务管理！")
    @GetMapping("/page")
    @ApiOperation("分页查询试验数据管理试验任务管理")
    @PreAuthorize("@el.check('select:tsTask')")
    public ResponseResult getTsTaskPage(String taskName, String startDate, String endDate, String taskPlace, String taskPerson, String taskCode, String taskType, String carrierName, String deviceCode, String testDescribe, String sensorDescribe, Page<TsTask> page, String attributeContentJson) {
        // 双重解码处理
        if (attributeContentJson != null) {
            try {
                // 第二次解码：%5B → [
                attributeContentJson = URLDecoder.decode(attributeContentJson, StandardCharsets.UTF_8.name());
            } catch (UnsupportedEncodingException e) {
                return ResponseResult.error("参数解码错误");
            }
        }

        // 将JSON字符串转换为List<Map<String, String>>
        List<Map<String, String>> attributeContent = null;
        if (StringUtils.isNotEmpty(attributeContentJson)) {
            try {
                // 使用 TypeReference 指定精确类型
                attributeContent = JSON.parseObject(
                        attributeContentJson,
                        new TypeReference<List<Map<String, String>>>() {
                        }
                );

                // 遍历并移除空值字段
                if (attributeContent != null) {
                    for (Map<String, String> item : attributeContent) {
                        // 遍历 Map，移除值为空的键值对
                        item.entrySet().removeIf(entry -> entry.getValue() == null || entry.getValue().isEmpty());
                    }
                }
            } catch (Exception e) {
                return ResponseResult.error("属性参数格式错误");
            }
        }

        //分页查询
        Page<TsTask> sdProjectPage = tsTaskService.getTsTaskPage(taskName, startDate, endDate, taskPlace, taskPerson, taskCode, taskType, carrierName, deviceCode, testDescribe, sensorDescribe, page, attributeContent);
        return ResponseResult.successData(sdProjectPage);
    }

    /***********************************
     * 用途说明：新增试验数据管理-试验任务管理
     * 参数说明
     * TsTask     试验任务管理
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Log(module = "试验数据管理", value = "新增试验数据管理试验任务管理！")
    @PostMapping("/addtsTask")
    @ApiOperation("新增试验数据管理试验任务管理")
    @ResponseBody
    @PreAuthorize("@el.check('add:tsTask')")
    public ResponseResult addtsTask(@RequestBody TsTask tsTask) throws IOException {
        //对象不能为空
        if (ObjUtil.isEmpty(tsTask)) {
            return ResponseResult.error("参数为空");
        }
        Boolean isOk = tsTaskService.addtsTask(tsTask);
        if (isOk) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    /**********************************
     * 用途说明: 修改试验数据管理-试验任务管理
     * 参数说明
     * TsTask     试验任务管理
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Log(module = "试验数据管理", value = "修改试验数据管理试验任务管理！")
    @PostMapping("/updatetsTask")
    @ApiOperation("修改试验数据管理试验任务管理")
    @PreAuthorize("@el.check('update:tsTask')")
    public ResponseResult updatetsTask(@RequestBody TsTask tsTask) throws IOException {
        //对象不能为空
        if (ObjUtil.isEmpty(tsTask) && StrUtil.isBlank(tsTask.getId())) {
            return ResponseResult.error("参数为空");
        }
        boolean isOk = tsTaskService.updatetsTask(tsTask);
        if (isOk) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    /**********************************
     * 用途说明: 根据ID试验数据管理-试验任务管理
     * 参数说明  id 试验数据管理ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回删除成功或者失败
     ***********************************/
    @Log(module = "试验数据管理", value = "根据ID删除试验数据管理试验任务管理")
    @PostMapping("/deleteTsTaskById")
    @ApiOperation("根据ID删除试验数据管理试验任务管理")
    @PreAuthorize("@el.check('del:tsTask')")
    public ResponseResult deleteTsTaskById(@RequestParam String id) {
        if (StrUtil.isBlank(id)) {
            return ResponseResult.error("参数为空");
        }
        List<String> dataset = new ArrayList<>();
        dataset.add(id);
        boolean isOk = tsTaskService.deleteTstaskByIds(dataset);
        if (isOk) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    /**********************************
     * 用途说明: 批量删除试验数据管理-试验任务管理
     * 参数说明  ids 试验数据管理id数组
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回批量删除成功或失败
     ***********************************/
    @Log(module = "试验数据管理", value = "批量删除试验数据管理试验任务管理！")
    @PostMapping("/deleteTsTaskByIds")
    @ApiOperation("批量删除试验数据管理试验任务管理")
    @PreAuthorize("@el.check('del:tsTask')")
    public ResponseResult deleteTsTaskByIds(@RequestParam String ids) {
        if (StrUtil.isBlank(ids)) {
            return ResponseResult.error("参数为空");
        }
        String[] splitIds = ids.split(",");
        // 数组转集合
        List<String> dataset = Arrays.asList(splitIds);
        boolean isOk = tsTaskService.deleteTstaskByIds(dataset);
        if (isOk) {
            return ResponseResult.success();
        } else {
            return ResponseResult.error();
        }
    }

    /**********************************
     * 用途说明: 根据ID确认是否可以实验任务项目
     * 参数说明  id 试验数据管理ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回删除成功或者失败
     ***********************************/
    @Log(module = "确认是否可以删除", value = "根据ID确认是否可以实验任务")
    @PostMapping("/confirmDeleteTask")
    @ApiOperation("根据ID确认是否可以实验任务")
    public ResponseResult confirmDeleteTask(@RequestParam String ids) {
        if (StrUtil.isBlank(ids)) {
            return ResponseResult.error("参数为空");
        }
        String[] splitIds = ids.split(",");
        // 数组转集合
        List<String> dataset = Arrays.asList(splitIds);
        return ResponseResult.successData(tsTaskService.confirmDeleteTask(dataset));
    }

    /***********************************
     * 用途说明： 查询所有试验数据管理-试验任务管理
     * 参数说明
     * 返回值说明:  试验数据管理试验任务管理
     ***********************************/
    @PostMapping("/list")
    @ApiOperation("查询所有试验数据管理试验任务管理！")
    @ResponseBody
    //@PreAuthorize("@el.check('select:devicesignal')")
    public ResponseResult listTsTask() {
        List<TsTask> tsTasks = tsTaskService.listTsTask();
        return ResponseResult.successData(tsTasks);
    }
}
