package com.yfd.platform.modules.experimentalData.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.experimentalData.domain.TsFiles;
import com.yfd.platform.modules.experimentalData.domain.TsNodes;
import com.yfd.platform.modules.experimentalData.domain.TsTask;
import com.yfd.platform.modules.experimentalData.mapper.TsFilesMapper;
import com.yfd.platform.modules.experimentalData.mapper.TsNodesMapper;
import com.yfd.platform.modules.experimentalData.mapper.TsTaskMapper;
import com.yfd.platform.modules.experimentalData.service.ITsFilesService;
import com.yfd.platform.modules.experimentalData.service.ITsNodesService;
import com.yfd.platform.modules.experimentalData.service.ITsTaskService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.specialDocument.domain.Project;
import com.yfd.platform.modules.storage.context.StorageSourceContext;
import com.yfd.platform.modules.storage.mapper.StorageSourceMapper;
import com.yfd.platform.modules.storage.model.entity.StorageSource;
import com.yfd.platform.modules.storage.model.entity.StorageSourceConfig;
import com.yfd.platform.modules.storage.model.enums.FileTypeEnum;
import com.yfd.platform.modules.storage.model.request.BatchDeleteRequest;
import com.yfd.platform.modules.storage.model.request.NewFolderRequest;
import com.yfd.platform.modules.storage.service.base.AbstractBaseFileService;
import com.yfd.platform.utils.StringUtils;
import com.yfd.platform.utils.TableNameContextHolder;
import io.netty.channel.ChannelInboundHandlerAdapter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.sql.*;
import java.time.format.DateTimeFormatter;
import java.util.*;


import javax.annotation.Resource;
import javax.sql.DataSource;
import java.time.LocalDateTime;
import java.util.stream.Collectors;

/**
 * <p>
 * 试验任务表 服务实现类
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-21
 */
@Service
public class TsTaskServiceImpl extends ServiceImpl<TsTaskMapper, TsTask> implements ITsTaskService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ChannelInboundHandlerAdapter.class);


    //试验任务Mapper
    @Resource
    private TsTaskMapper tsTaskMapper;

    //实验数据管理
    @Resource
    private ITsNodesService tsNodesService;

    @Resource
    private ITsFilesService tsFilesService;

    @Resource
    private TsNodesMapper tsNodesMapper;

    @Resource
    private TsFilesMapper tsFilesMapper;

    @Resource
    private StorageSourceContext storageSourceContext;

    @Resource
    private StorageSourceMapper storageSourceMapper;

    @Autowired
    private DataSource dataSource;

    private static final String INITIAL_CODE = "00001";
    private static final int MAX_CODE_VALUE = 99999;

    /**********************************
     * 用途说明: 分页查询试验数据管理-试验任务管理
     * 参数说明
     * taskName      任务名称
     * startDate     (开始日期)
     * endDate      （结束日期）
     * taskPlace     任务地点
     * taskPerson    任务人员
     * taskCode      任务编号
     * taskType      任务类型
     * carrierName    载体名称
     * deviceCode    设备代号_编号
     * testDescribe    试验描述
     * sensorDescribe    传感器描述
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public Page<TsTask> getTsTaskPage(String taskName, String startDate, String endDate, String taskPlace, String taskPerson, String taskCode, String taskType, String carrierName, String deviceCode, String testDescribe, String sensorDescribe, Page<TsTask> page,List<Map<String, String>> attributeContent) {

        LambdaQueryWrapper<TsTask> queryWrapper = new LambdaQueryWrapper<>();
        //如果任务名称    taskName 不为空
        if (StringUtils.isNotEmpty(taskName)) {
            queryWrapper.like(TsTask::getTaskName, taskName);
        }

        //如果任务地点   taskPlace 不为空
        if (StringUtils.isNotEmpty(taskPlace)) {
            queryWrapper.like(TsTask::getTaskPlace, taskPlace);
        }

        //如果任务人员  taskPerson 不为空
        if (StringUtils.isNotEmpty(taskPerson)) {
            queryWrapper.like(TsTask::getTaskPerson, taskPerson);
        }

        //如果任务编号  taskCode 不为空
        if (StringUtils.isNotEmpty(taskCode)) {
            queryWrapper.like(TsTask::getTaskCode, taskCode);
        }

        //如果任务类型  taskType 不为空
        if (StringUtils.isNotEmpty(taskType)) {
            queryWrapper.eq(TsTask::getTaskType, taskType);
        }
        //如果载体名称  carrierName 不为空
        if (StringUtils.isNotEmpty(carrierName)) {
            queryWrapper.like(TsTask::getCarrierName, carrierName);
        }
        //如果设备代号_编号  deviceCode 不为空
        if (StringUtils.isNotEmpty(deviceCode)) {
            queryWrapper.like(TsTask::getDeviceCode, deviceCode);
        }
        //如果试验描述  testDescribe 不为空
        if (StringUtils.isNotEmpty(testDescribe)) {
            queryWrapper.like(TsTask::getTestDescribe, testDescribe);
        }
        //如果传感器描述  sensorDescribe 不为空
        if (StringUtils.isNotEmpty(sensorDescribe)) {
            queryWrapper.like(TsTask::getSensorDescribe, sensorDescribe);
        }


        //开始时间startDate
        DateTime parseStartDate = DateUtil.parse(startDate);
        //时间endDate不为空
        DateTime parseEndDate = DateUtil.parse(endDate);
        //开始时间和结束时间不为空    查询条件>=开始时间  <结束时间
        if (parseStartDate != null && parseEndDate != null) {
            queryWrapper.ge(TsTask::getTaskStartdate, parseStartDate).lt(TsTask::getTaskEnddate, parseEndDate);
        }
        queryWrapper.orderByDesc(TsTask::getTaskStartdate);


        // 处理属性过滤条件 - MySQL 5.7+ 兼容版本
        if (attributeContent != null && !attributeContent.isEmpty()) {
            for (Map<String, String> attr : attributeContent) {
                for (Map.Entry<String, String> entry : attr.entrySet()) {
                    String code = entry.getKey();
                    String value = entry.getValue();

                    if (StringUtils.isEmpty(value)) {
                        // 检查属性存在
                        queryWrapper.apply(
                                "EXISTS (SELECT 1 FROM JSON_TABLE(task_props, '$[*]' " +
                                        "COLUMNS (code VARCHAR(50) PATH '$.code') AS jt " +
                                        "WHERE jt.code = {0})",
                                code
                        );
                    } else {
                        // 转义特殊字符
                        String safeValue = value.replace("'", "''")
                                .replace("%", "\\%")
                                .replace("_", "\\_");

                        // 使用 JSON_EXTRACT 实现兼容查询
                        queryWrapper.apply(
                                "EXISTS (SELECT 1 FROM (" +
                                        "   SELECT " +
                                        "       JSON_UNQUOTE(JSON_EXTRACT(t.obj, '$.code')) AS code, " +
                                        "       JSON_UNQUOTE(JSON_EXTRACT(t.obj, '$.data')) AS data " +
                                        "   FROM (" +
                                        "       SELECT JSON_EXTRACT(task_props, CONCAT('$[', idx.idx, ']')) AS obj " +
                                        "       FROM (SELECT 0 AS idx UNION SELECT 1 UNION SELECT 2 UNION SELECT 3) idx" +
                                        "       WHERE idx.idx < JSON_LENGTH(task_props)" +
                                        "   ) t" +
                                        ") jt " +
                                        "WHERE jt.code = {0} AND jt.data LIKE CONCAT('%', {1}, '%'))",
                                code, value
                        );
                    }
                }
            }
        }
        //分页查询
        Page<TsTask> tsTaskPage = tsTaskMapper.selectPage(page, queryWrapper);
        return tsTaskPage;
    }

    /***********************************
     * 用途说明：新增试验数据管理-试验任务管理
     * 参数说明
     * TsTask     试验任务管理
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    public Boolean addtsTask(TsTask tsTask) throws IOException {
        //todo  新增实验任务的时候创建一个本地的文件夹
        //生成任务编号
        String taskCode = generateNextTsTaskCode();
        tsTask.setTaskCode(taskCode);

        //处理属性
        String frontEndJson = tsTask.getTaskProps();
        List<TsTask> tsTasksList = tsTaskMapper.selectList(new LambdaQueryWrapper<>());
        if (tsTasksList.size() > 0) {
            for (TsTask tsTaskData : tsTasksList) {
                String dbJson = tsTaskData.getTaskProps();
                String syncData = syncData(frontEndJson, dbJson);
                tsTaskData.setTaskProps(syncData);
                tsTaskMapper.updateById(tsTaskData);
            }
        }


        //生成任务名称   任务开始时间_结束时间_地点_载机名称_设备代号_编号
        String taskName = buildTaskName(tsTask);
        tsTask.setTaskName(taskName);
        LambdaQueryWrapper<TsTask> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TsTask::getTaskName, tsTask.getTaskName());
        TsTask tsTask1 = tsTaskMapper.selectOne(queryWrapper);
        if (tsTask1 != null) {
            throw new RuntimeException("试验任务管理项目已存在！");
        }

        StorageSource storageSource = getStorageConfig(tsTask.getLocalStorageId());

        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        tsTask.setCreateTime(currentTime);
        int valueAdded = tsTaskMapper.insert(tsTask);
        if (valueAdded == 1) {

            // 根节点路径
            String path = "/";
            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(tsTask.getTaskName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(path);//请求路径,示例值(/)
            newFolderRequest.setStorageKey(storageSource.getKey());//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {
                // 动态创建数据表
                createTaskFileTable(taskCode);
                return true;
            } else {
                LOGGER.error("试验任务增加成功，但是本地试验任务文件夹创建失败");
                return false;
            }
        } else {
            return false;
        }
    }

    public static String syncData(String frontEndJson, String dbJson) throws IOException {
        // 1. 如果 frontEndJson 为空，则返回一个空的 dbJson
        if (frontEndJson == null || frontEndJson.trim().isEmpty()) {
            return "[]"; // 返回空的 JSON 数组
        }

        // 2. 解析前端传递的 JSON 字符串为 List<Map<String, String>>，如果为空，直接返回空的 dbJson
        ObjectMapper objectMapper = new ObjectMapper();
        List<Map<String, String>> frontEndData = objectMapper.readValue(frontEndJson, List.class);

        // 3. 如果前端有数据，且 dbJson 为空，直接使用前端数据并将 data 设置为空
        if (frontEndData != null && !frontEndData.isEmpty() && (dbJson == null || dbJson.trim().isEmpty())) {
            List<Map<String, String>> result = new ArrayList<>();
            for (Map<String, String> item : frontEndData) {
                Map<String, String> newItem = new HashMap<>();
                newItem.put("code", item.get("code"));
                newItem.put("name", item.get("name"));
                newItem.put("data", ""); // 数据为空
                result.add(newItem);
            }
            return objectMapper.writeValueAsString(result); // 返回填充后的结果
        }

        // 4. 如果 dbJson 不为空，解析 dbJson
        List<Map<String, String>> dbData = objectMapper.readValue(dbJson, List.class);

        // 5. 根据前端数据创建一个Map，以便快速查找前端的项
        Map<String, Map<String, String>> frontEndMap = frontEndData.stream()
                .collect(Collectors.toMap(item -> (String) item.get("name"), item -> item));

        // 6. 遍历数据库中的数据，进行相应的增、删、改
        List<Map<String, String>> result = new ArrayList<>();

        // 7. 处理数据库中的项：先将前端数据对应的项保留，新增的项加上空的数据
        for (Map<String, String> dbItem : dbData) {
            String name = dbItem.get("name");
            if (frontEndMap.containsKey(name)) {
                // 如果前端有该项，则保留数据库中的项，保持其原有的id
                // 只有当前端的 data 不为空时才更新，避免改变数据库中原有的 data
                String frontEndDataValue = frontEndMap.get(name).get("data");
//                if (frontEndDataValue != null && !frontEndDataValue.trim().isEmpty()) {
//                    //dbItem.put("data", frontEndDataValue); // 只在前端有值时才更新
//                }
                result.add(dbItem);
                frontEndMap.remove(name); // 从前端map中移除已处理的项
            }
        }

        // 8. 处理前端数据中没有在数据库中出现的项：新增这些项，data为空
        for (Map.Entry<String, Map<String, String>> entry : frontEndMap.entrySet()) {
            Map<String, String> newItem = new HashMap<>();
            newItem.put("code", entry.getValue().get("code")); // 保持前端传来的id
            newItem.put("name", entry.getValue().get("name"));
            newItem.put("data", ""); // 如果数据库中没有该项，则data为空
            result.add(newItem);
        }

        // 9. 将结果转换为 JSON 字符串并返回
        return objectMapper.writeValueAsString(result);
    }


    // 动态创建任务文件表
    private void createTaskFileTable(String taskCode) {

        // 添加表名校验（重要！防止SQL注入）
        if (!taskCode.matches("^\\d{5}$")) { // 假设任务编号是5位数字
            throw new IllegalArgumentException("非法任务编号格式: " + taskCode);
        }

        String tableName = "ts_files_" + taskCode;
        String sql = String.format(
                "CREATE TABLE IF NOT EXISTS `%s` LIKE `ts_files`",
                tableName
        );
        try (Connection conn = dataSource.getConnection();
             Statement stmt = conn.createStatement()) {
            // 1. 记录要执行的SQL
            LOGGER.info("执行DDL: {}", sql);
            // 2. 执行SQL
            boolean result = stmt.execute(sql);
            // 3. 验证结果
            LOGGER.debug("执行结果: {}", result);
            // 4. 二次验证
            if (!tableExists(tableName)) {
                throw new RuntimeException("表创建后验证失败: " + tableName);
            }

        } catch (SQLException e) {
            // 提取MySQL错误信息
            String errorMsg = extractMysqlError(e);
            LOGGER.error("创建表失败: {} - {}", tableName, errorMsg);
            throw new RuntimeException("创建表失败: " + errorMsg, e);
        }
    }

    private boolean tableExists(String tableName) {
        try (Connection conn = dataSource.getConnection()) {
            DatabaseMetaData meta = conn.getMetaData();
            // 获取当前数据库信息
            String catalog = conn.getCatalog();
            String schema = conn.getSchema();
            // 查询表是否存在
            try (ResultSet rs = meta.getTables(catalog, schema, tableName, new String[]{"TABLE"})) {
                return rs.next();
            }
        } catch (SQLException e) {
            LOGGER.error("检查表存在失败: {}", tableName, e);
            return false;
        }
    }

    private String extractMysqlError(SQLException e) {
        // 提取MySQL特有的错误信息
        while (e != null) {
            if (e.getMessage().contains("MySQL")) {
                return String.format("MySQL 错误 [%d]: %s", e.getErrorCode(), e.getMessage());
            }
            e = e.getNextException();
        }
        return "未知数据库错误";
    }

    public static String buildTaskName(TsTask tsTask) {
        List<String> parts = new ArrayList<>();
        // 定义日期格式化模板
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd");
        // 按顺序添加非空字段
        addIfNotEmpty(parts, formatDate(tsTask.getTaskStartdate(), formatter));
        addIfNotEmpty(parts, formatDate(tsTask.getTaskEnddate(), formatter));
        addIfNotEmpty(parts, tsTask.getTaskPlace());
        addIfNotEmpty(parts, tsTask.getCarrierName());
        addIfNotEmpty(parts, tsTask.getDeviceCode());

        // 使用下划线连接非空部分
        return String.join("_", parts);
    }

    // 格式化日期的方法
    private static String formatDate(Object date, DateTimeFormatter formatter) {
        if (date != null) {
            return ((java.time.LocalDate) date).format(formatter);
        }
        return null;
    }


    private static void addIfNotEmpty(List<String> list, String value) {
        if (StringUtils.isNotBlank(value)) { // 使用Apache Commons Lang的空判断
            list.add(value.trim()); // 去除首尾空格
        }
    }

    // 备用方案（不使用第三方库）
    private static boolean isNotEmpty(String value) {
        return value != null && !value.trim().isEmpty();
    }

    /**********************************
     * 用途说明: 修改试验数据管理-试验任务管理
     * 参数说明
     * TsTask     试验任务管理
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    public boolean updatetsTask(TsTask tsTask) throws IOException {
        Boolean value = false;

        //处理属性
        String frontEndJson = tsTask.getTaskProps();
        List<TsTask> tsTasksList = tsTaskMapper.selectList(new LambdaQueryWrapper<>());
        if (tsTasksList.size() > 0) {
            for (TsTask tsTaskData : tsTasksList) {
                String dbJson = tsTaskData.getTaskProps();
                String syncData = syncData(frontEndJson, dbJson);
                tsTaskData.setTaskProps(syncData);
                tsTaskMapper.updateById(tsTaskData);
            }
        }

        //修改的时候判断本地存储空间是不是发生了变化  如果是的话 需要重新创建本地文件
        TsTask TsTaskOld = tsTaskMapper.selectById(tsTask.getId());
        StorageSource storageSource = getStorageConfig(tsTask.getLocalStorageId());


        if (!TsTaskOld.getLocalStorageId().equals(tsTask.getLocalStorageId())) {
            String path = "/";
            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(tsTask.getTaskName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(path);//请求路径,示例值(/)
            newFolderRequest.setStorageKey(storageSource.getKey());//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {
                String taskName = buildTaskName(tsTask);
                tsTask.setTaskName(taskName);
                int valueUpdate = tsTaskMapper.updateById(tsTask);
                if (valueUpdate == 1) {
                    value = true;
                } else {
                    value = false;
                }
                value = true;
            } else {
                LOGGER.error("节点新增成功，但是本地专项文件夹创建失败");
                value = false;
            }
        } else {

            String taskName = buildTaskName(tsTask);
            tsTask.setTaskName(taskName);
            int valueUpdate = tsTaskMapper.updateById(tsTask);
            if (valueUpdate == 1) {
                value = true;
            } else {
                value = false;
            }

        }
        return value;
    }

    /**********************************
     * 用途说明: 批量删除试验数据管理-试验任务管理
     * 参数说明  ids 试验数据管理id数组
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回批量删除成功或失败
     ***********************************/
    @Override
    public boolean deleteTstaskByIds(List<String> dataset) {
        Boolean value = false;

        try {


            //循环所有的ID
            for (String taskId : dataset) {
                //删除项目
                TsTask tsTask = tsTaskMapper.selectById(taskId);
                TableNameContextHolder.setTaskCode(tsTask.getTaskCode());
                // 删除节点表
                LambdaQueryWrapper<TsNodes> deleteWrapper = new LambdaQueryWrapper<>();
                deleteWrapper.eq(TsNodes::getTaskId, taskId);
                tsNodesMapper.delete(deleteWrapper);

                // 删除文件表
                LambdaQueryWrapper<TsFiles> deleteWrapperFiles = new LambdaQueryWrapper<>();
                deleteWrapperFiles.eq(TsFiles::getTaskId, taskId);
                tsFilesMapper.delete(deleteWrapperFiles);
                //todo 删除文件表数据
//            tsFilesMapper.deleteSdFilesBytaskId(taskId);


//            String path = "/" + tsTask.getTaskName() + "/";
//            //调用删除节点 根据任务ID
//            Boolean deleteTsnodes = tsNodesService.deleteTsNodesByTaskId(taskId, path);
//            //如果删除成功 接着删除节点表数据
//            if (deleteTsnodes) {
//                LOGGER.info("tsNodes表结删除改成功");
//                value = true;
//            } else {
//                LOGGER.error("tsNodes表结构删除失败");
//                value = false;
//            }

                StorageSource storageSource = getStorageConfig(tsTask.getLocalStorageId());
                // 删除 local 中的文件夹  项目文件夹
                List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
                BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
                deleteItemData.setName(tsTask.getTaskName());
                deleteItemData.setPassword("");
                deleteItemData.setPath("/");
                deleteItemData.setType(FileTypeEnum.FOLDER);
                deleteItemList.add(deleteItemData);

                BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
                batchDeleteRequest.setDeleteItems(deleteItemList);
                batchDeleteRequest.setStorageKey(storageSource.getKey());
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
                int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItemList);

                for (BatchDeleteRequest.DeleteItem deleteItem : deleteItemList) {
                    boolean flag = false;
                    try {
                        if (deleteItem.getType() == FileTypeEnum.FILE) {
                            flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                        } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                            flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                        }

                        if (flag) {
                            deleteSuccessCount++;
                        } else {
                            deleteFailCount++;
                        }
                    } catch (Exception e) {
                        LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                        deleteFailCount++;
                    }
                }

                if (deleteSuccessCount >= 1) {
                    // 删除当前项目
                    int deleteCount = tsTaskMapper.deleteById(taskId);
                    if (deleteCount == 1) {
                        LOGGER.info("tstask表结删除改成功");
                        value = true;
                    } else {
                        LOGGER.error("tstask表结构删除失败");
                        value = false;
                    }
                } else {
                    value = false;
                }


            }

            return value;
        } catch (Exception e) {

        } finally {
            TableNameContextHolder.clear();
        }
        return value;
    }

    /**********************************
     * 用途说明: 根据ID确认是否可以实验任务项目
     * 参数说明  id 试验数据管理ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回删除成功或者失败
     ***********************************/
    @Override
    public Object confirmDeleteTask(List<String> dataset) {
        try {
            LambdaQueryWrapper<TsTask> queryWrapperTsk = new LambdaQueryWrapper<>();
            queryWrapperTsk.in(TsTask::getId, dataset);
            List<TsTask> tsTasks = tsTaskMapper.selectList(queryWrapperTsk);

            JSONObject result = new JSONObject();
            boolean allDeletable = true;

            for (TsTask tsTask : tsTasks) {
                TableNameContextHolder.setTaskCode(tsTask.getTaskCode());

                // 查询是否有备份路径或非空路径的文件
                LambdaQueryWrapper<TsFiles> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(TsFiles::getTaskId, tsTask.getId())
                        .and(wrapper -> wrapper.isNotNull(TsFiles::getBackupPath)
                                .or().ne(TsFiles::getBackupPath, ""));

                int count = tsFilesService.count(queryWrapper);

                if (count > 0) {
                    allDeletable = false;
                    break; // 一旦发现不可删除项，提前终止循环
                }
            }

            result.putOpt("status", allDeletable ? "1" : "0");
            return ResponseResult.successData(result);

        } catch (Exception e) {
            // 使用日志框架代替 printStackTrace
            LOGGER.error("确认删除实验任务时发生异常", e);
            return ResponseResult.error("查询失败");
        } finally {
            TableNameContextHolder.clear();
        }
    }

    @Override
    public List<TsTask> listTsTask() {
        LambdaQueryWrapper<TsTask> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.orderByDesc(TsTask::getTaskStartdate);
        List<TsTask> tsTasks = tsTaskMapper.selectList(queryWrapper);
        for (TsTask tsTask : tsTasks) {
            StorageSource storageSource = getStorageConfig(tsTask.getLocalStorageId());
            tsTask.setKey(storageSource.getKey());
        }
        return tsTasks;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized String generateNextTsTaskCode() {
        // 1. 使用悲观锁查询最大编号
        TsTask maxTsTask = this.getOne(
                new QueryWrapper<TsTask>()
                        .select("MAX(task_code) AS taskCode")
                        .last("FOR UPDATE")
        );


        // 2. 处理空表情况
        String currentMax = (maxTsTask != null && maxTsTask.getTaskCode() != null)
                ? maxTsTask.getTaskCode()
                : INITIAL_CODE;

        // 3. 验证并生成新编号
        try {
            int numericCode = Integer.parseInt(currentMax);
            if (numericCode < 0 || numericCode > MAX_CODE_VALUE) {
                throw new IllegalStateException("当前设备编号超出有效范围: " + currentMax);
            }
            if (numericCode == MAX_CODE_VALUE) {
                throw new IllegalStateException("设备编号已达最大值" + MAX_CODE_VALUE);
            }
            return String.format("%05d", numericCode + 1);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("无效的设备编号格式: " + currentMax);
        }

    }

    // 辅助方法：获取存储配置
    private StorageSource getStorageConfig(Integer id) {
        return storageSourceMapper.selectOne(new LambdaQueryWrapper<StorageSource>().eq(StorageSource::getId, id)
        );
    }
}
