package com.yfd.platform.modules.specialDocument.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yfd.platform.modules.specialDocument.domain.Nodes;
import com.yfd.platform.modules.specialDocument.domain.Project;
import com.yfd.platform.modules.specialDocument.mapper.ProjectMapper;
import com.yfd.platform.modules.specialDocument.service.INodesService;
import com.yfd.platform.modules.specialDocument.service.IProjectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.storage.context.StorageSourceContext;
import com.yfd.platform.modules.storage.mapper.StorageSourceMapper;
import com.yfd.platform.modules.storage.model.entity.StorageSource;
import com.yfd.platform.modules.storage.model.enums.FileTypeEnum;
import com.yfd.platform.modules.storage.model.request.BatchDeleteRequest;
import com.yfd.platform.modules.storage.model.request.NewFolderRequest;
import com.yfd.platform.modules.storage.service.base.AbstractBaseFileService;
import com.yfd.platform.utils.StringUtils;
import io.netty.channel.ChannelInboundHandlerAdapter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.io.IOException;
import java.sql.*;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 专项项目表 服务实现类
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-20
 */
@Service
public class ProjectServiceImpl extends ServiceImpl<ProjectMapper, Project> implements IProjectService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ChannelInboundHandlerAdapter.class);


    private static final String INITIAL_CODE = "00001";
    private static final int MAX_CODE_VALUE = 99999;


    //专项项目表Mapper
    @Resource
    private ProjectMapper projectMapper;


    //专项文档节点服务类
    @Resource
    private INodesService nodesService;


    @Resource
    private StorageSourceContext storageSourceContext;

    @Resource
    private StorageSourceMapper storageSourceMapper;

    @Autowired
    private DataSource dataSource;

    /**********************************
     * 用途说明: 分页查询专项文档管理-项目管理
     * 参数说明
     * description   项目描述
     * projectType   项目类型
     *projectName    项目名称
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public Page<Project> getSdProjectPage(String description, String projectType, String projectName, Page<Project> page,List<Map<String, String>> attributeContent) {

        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();

        // 基础条件处理
        if (StringUtils.isNotEmpty(description))
            queryWrapper.like(Project::getDescription, description);
        if (StringUtils.isNotEmpty(projectType))
            queryWrapper.like(Project::getProjectType, projectType);
        if (StringUtils.isNotEmpty(projectName))
            queryWrapper.like(Project::getProjectName, projectName);
        queryWrapper.orderByDesc(Project::getProjectTime);
        // 处理属性过滤条件 - MySQL 5.7+ 兼容版本
        if (attributeContent != null && !attributeContent.isEmpty()) {
            for (Map<String, String> attr : attributeContent) {
                for (Map.Entry<String, String> entry : attr.entrySet()) {
                    String code = entry.getKey();
                    String value = entry.getValue();

                    if (StringUtils.isEmpty(value)) {
                        // 检查属性存在
                        queryWrapper.apply(
                                "EXISTS (SELECT 1 FROM JSON_TABLE(project_props, '$[*]' " +
                                        "COLUMNS (code VARCHAR(50) PATH '$.code') AS jt " +
                                        "WHERE jt.code = {0})",
                                code
                        );
                    } else {
                        // 转义特殊字符
                        String safeValue = value.replace("'", "''")
                                .replace("%", "\\%")
                                .replace("_", "\\_");

                        // 使用 JSON_EXTRACT 实现兼容查询
                        queryWrapper.apply(
                                "EXISTS (SELECT 1 FROM (" +
                                        "   SELECT " +
                                        "       JSON_UNQUOTE(JSON_EXTRACT(t.obj, '$.code')) AS code, " +
                                        "       JSON_UNQUOTE(JSON_EXTRACT(t.obj, '$.data')) AS data " +
                                        "   FROM (" +
                                        "       SELECT JSON_EXTRACT(project_props, CONCAT('$[', idx.idx, ']')) AS obj " +
                                        "       FROM (SELECT 0 AS idx UNION SELECT 1 UNION SELECT 2 UNION SELECT 3) idx" +
                                        "       WHERE idx.idx < JSON_LENGTH(project_props)" +
                                        "   ) t" +
                                        ") jt " +
                                        "WHERE jt.code = {0} AND jt.data LIKE CONCAT('%', {1}, '%'))",
                                code, value
                        );
                    }
                }
            }
        }
        return projectMapper.selectPage(page, queryWrapper);

    }

    /***********************************
     * 用途说明：新增专项文档管理-项目管理
     * 参数说明
     * Project     项目管理
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    public Boolean addSdproject(Project project) throws IOException {
        //创建项目的时候要创建一个本地的文件夹 路径的话去找专项的存储路径
        //生成项目编号
        String projectCode = generateNextProjectCode();
        project.setProjectCode(projectCode);

        String frontEndJson = project.getProjectProps();
        List<Project> projectList = projectMapper.selectList(new LambdaQueryWrapper<>());
        if (projectList.size()>0){
            for (Project projectData : projectList) {
                String dbJson = projectData.getProjectProps();
                String syncData = syncData(frontEndJson, dbJson);
                projectData.setProjectProps(syncData);
                projectMapper.updateById(projectData);
            }
        }


        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        project.setCreateTime(currentTime);
        StorageSource storageSource = getStorageConfig(project.getLocalStorageId());

        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Project::getProjectName,project.getProjectName());
        Project project1 = projectMapper.selectOne(queryWrapper);
        if (project1 != null){
            throw new RuntimeException("专项文档管理项目已存在！");
        }


        int valueAdded = projectMapper.insert(project);
        if (valueAdded == 1) {

            // 根节点路径
            String path = "/";
            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(project.getProjectName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(path);//请求路径,示例值(/)
            newFolderRequest.setStorageKey(storageSource.getKey());//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {

                // 动态创建数据表
                //createTaskFileTable(projectCode);
                return true;
            } else {
                LOGGER.error("节点新增成功，但是本地专项文件夹创建失败");
                return false;
            }

        } else {
            return false;
        }

    }

    // 动态创建任务文件表
    private void createTaskFileTable(String taskCode) {

        // 添加表名校验（重要！防止SQL注入）
        if (!taskCode.matches("^\\d{5}$")) { // 假设任务编号是5位数字
            throw new IllegalArgumentException("非法任务编号格式: " + taskCode);
        }

        String tableName = "sd_files_" + taskCode;
        String sql = String.format(
                "CREATE TABLE IF NOT EXISTS `%s` LIKE `sd_files`",
                tableName
        );
        try (Connection conn = dataSource.getConnection();
             Statement stmt = conn.createStatement()) {
            // 1. 记录要执行的SQL
            LOGGER.info("执行DDL: {}", sql);
            // 2. 执行SQL
            boolean result = stmt.execute(sql);
            // 3. 验证结果
            LOGGER.debug("执行结果: {}", result);
            // 4. 二次验证
            if (!tableExists(tableName)) {
                throw new RuntimeException("表创建后验证失败: " + tableName);
            }

        } catch (SQLException e) {
            // 提取MySQL错误信息
            String errorMsg = extractMysqlError(e);
            LOGGER.error("创建表失败: {} - {}", tableName, errorMsg);
            throw new RuntimeException("创建表失败: " + errorMsg, e);
        }
    }

    private boolean tableExists(String tableName) {
        try (Connection conn = dataSource.getConnection()) {
            DatabaseMetaData meta = conn.getMetaData();
            // 获取当前数据库信息
            String catalog = conn.getCatalog();
            String schema = conn.getSchema();
            // 查询表是否存在
            try (ResultSet rs = meta.getTables(catalog, schema, tableName, new String[]{"TABLE"})) {
                return rs.next();
            }
        } catch (SQLException e) {
            LOGGER.error("检查表存在失败: {}", tableName, e);
            return false;
        }
    }

    private String extractMysqlError(SQLException e) {
        // 提取MySQL特有的错误信息
        while (e != null) {
            if (e.getMessage().contains("MySQL")) {
                return String.format("MySQL 错误 [%d]: %s", e.getErrorCode(), e.getMessage());
            }
            e = e.getNextException();
        }
        return "未知数据库错误";
    }

    // 编号生成工具方法
    @Transactional(rollbackFor = Exception.class)
    public synchronized String generateNextProjectCode() {
        // 1. 使用悲观锁查询最大编号
        Project maxProject = this.getOne(
                new QueryWrapper<Project>()
                        .select("MAX(project_code) AS projectCode")
                        .last("FOR UPDATE")
        );

        // 2. 处理空表情况
        String currentMax = (maxProject != null && maxProject.getProjectCode() != null)
                ? maxProject.getProjectCode()
                : INITIAL_CODE;

        // 3. 验证并生成新编号
        try {
            int numericCode = Integer.parseInt(currentMax);
            if (numericCode < 0 || numericCode > MAX_CODE_VALUE) {
                throw new IllegalStateException("当前项目编号超出有效范围: " + currentMax);
            }
            if (numericCode == MAX_CODE_VALUE) {
                throw new IllegalStateException("项目编号已达最大值" + MAX_CODE_VALUE);
            }
            return String.format("%05d", numericCode + 1);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("无效的项目编号格式: " + currentMax);
        }

    }



    /**********************************
     * 用途说明: 修改专项文档管理-项目管理
     * 参数说明
     * Project     项目管理
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    public boolean updateSdproject(Project project) throws IOException {
        Boolean value = true;
        if (project.getProjectTime() == null || project.getProjectTime().equals("")) {
            project.setProjectTime(null);
        }
        String frontEndJson = project.getProjectProps();
        List<Project> projectList = projectMapper.selectList(new LambdaQueryWrapper<>());
        if (projectList.size()>0){
            for (Project projectData : projectList) {
                String dbJson = projectData.getProjectProps();
                String syncData = syncData(frontEndJson, dbJson);
                projectData.setProjectProps(syncData);
                projectMapper.updateById(projectData);
            }
        }



        //修改的时候判断本地存储空间是不是发生了变化  如果是的话 需要重新创建本地文件
        Project projectOld = projectMapper.selectById(project.getId());
        StorageSource storageSource = getStorageConfig(project.getLocalStorageId());
        if (!projectOld.getLocalStorageId().equals(project.getLocalStorageId())){
            String path = "/";
            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(project.getProjectName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(path);//请求路径,示例值(/)
            newFolderRequest.setStorageKey(storageSource.getKey());//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {
                int valueUpdate = projectMapper.updateById(project);
                if (valueUpdate == 1) {
                    value = true;
                } else {
                    value = false;
                }
                value = true;
            } else {
                LOGGER.error("节点新增成功，但是本地专项文件夹创建失败");
                value = false;
            }
        }else {
            int valueUpdate = projectMapper.updateById(project);
            if (valueUpdate == 1) {
                value = true;
            } else {
                value = false;
            }
        }
        return value;
    }

    public static String syncData(String frontEndJson, String dbJson) throws IOException {
        // 1. 如果 frontEndJson 为空，则返回一个空的 dbJson
        if (frontEndJson == null || frontEndJson.trim().isEmpty()) {
            return "[]"; // 返回空的 JSON 数组
        }

        // 2. 解析前端传递的 JSON 字符串为 List<Map<String, String>>，如果为空，直接返回空的 dbJson
        ObjectMapper objectMapper = new ObjectMapper();
        List<Map<String, String>> frontEndData = objectMapper.readValue(frontEndJson, List.class);

        // 3. 如果前端有数据，且 dbJson 为空，直接使用前端数据并将 data 设置为空
        if (frontEndData != null && !frontEndData.isEmpty() && (dbJson == null || dbJson.trim().isEmpty())) {
            List<Map<String, String>> result = new ArrayList<>();
            for (Map<String, String> item : frontEndData) {
                Map<String, String> newItem = new HashMap<>();
                newItem.put("code", item.get("code"));
                newItem.put("name", item.get("name"));
                newItem.put("data", ""); // 数据为空
                result.add(newItem);
            }
            return objectMapper.writeValueAsString(result); // 返回填充后的结果
        }

        // 4. 如果 dbJson 不为空，解析 dbJson
        List<Map<String, String>> dbData = objectMapper.readValue(dbJson, List.class);

        // 5. 根据前端数据创建一个Map，以便快速查找前端的项
        Map<String, Map<String, String>> frontEndMap = frontEndData.stream()
                .collect(Collectors.toMap(item -> (String) item.get("name"), item -> item));

        // 6. 遍历数据库中的数据，进行相应的增、删、改
        List<Map<String, String>> result = new ArrayList<>();

        // 7. 处理数据库中的项：先将前端数据对应的项保留，新增的项加上空的数据
        for (Map<String, String> dbItem : dbData) {
            String name = dbItem.get("name");
            if (frontEndMap.containsKey(name)) {
                // 如果前端有该项，则保留数据库中的项，保持其原有的id
                // 只有当前端的 data 不为空时才更新，避免改变数据库中原有的 data
                String frontEndDataValue = frontEndMap.get(name).get("data");
//                if (frontEndDataValue != null && !frontEndDataValue.trim().isEmpty()) {
//                    //dbItem.put("data", frontEndDataValue); // 只在前端有值时才更新
//                }
                result.add(dbItem);
                frontEndMap.remove(name); // 从前端map中移除已处理的项
            }
        }

        // 8. 处理前端数据中没有在数据库中出现的项：新增这些项，data为空
        for (Map.Entry<String, Map<String, String>> entry : frontEndMap.entrySet()) {
            Map<String, String> newItem = new HashMap<>();
            newItem.put("code", entry.getValue().get("code")); // 保持前端传来的id
            newItem.put("name", entry.getValue().get("name"));
            newItem.put("data", ""); // 如果数据库中没有该项，则data为空
            result.add(newItem);
        }

        // 9. 将结果转换为 JSON 字符串并返回
        return objectMapper.writeValueAsString(result);
    }

    /**********************************
     * 用途说明: 批量删除专项文档管理-项目管理
     * 参数说明  ids 项目管理id数组
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回批量删除成功或失败
     ***********************************/
    @Override
    public boolean deleteProjectByIds(List<String> dataset) {
        Boolean value = false;

        //循环所有的ID
        for (String id : dataset) {
            Project project = projectMapper.selectById(id);
            StorageSource storageSource = getStorageConfig(project.getLocalStorageId());

            String path = "/" + project.getProjectName() + "/";
            QueryWrapper<Nodes> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("project_id", id);
            queryWrapper.eq("parent_id", "00");
            List<Nodes> nodesList = nodesService.list(queryWrapper);
            //如果为空 跳过当前项目 接着下一个项目
            if (nodesList != null || !nodesList.isEmpty()) {
                for (Nodes nodes : nodesList) {
                    //调用删除节点 根据任务ID
                    Boolean deleteTsnodes = nodesService.deleteNodesById(nodes.getId(), path);
                    //如果删除成功 接着删除节点表数据
                    if (deleteTsnodes) {
                        LOGGER.info("nodes表结删除改成功");
                        value = true;
                    } else {
                        LOGGER.error("nodes表结构删除失败");
                        value = false;
                    }
                }
            }

            // 删除 sdlocal 中的文件夹  项目文件夹
            List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
            BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
            deleteItemData.setName(project.getProjectName());
            deleteItemData.setPassword("");
            deleteItemData.setPath("/");
            deleteItemData.setType(FileTypeEnum.FOLDER);
            deleteItemList.add(deleteItemData);

            BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
            batchDeleteRequest.setDeleteItems(deleteItemList);
            batchDeleteRequest.setStorageKey(storageSource.getKey());
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
            int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItemList);

            for (BatchDeleteRequest.DeleteItem deleteItem : deleteItemList) {
                boolean flag = false;
                try {
                    if (deleteItem.getType() == FileTypeEnum.FILE) {
                        flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                    } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                        flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                    }

                    if (flag) {
                        deleteSuccessCount++;
                    } else {
                        deleteFailCount++;
                    }
                } catch (Exception e) {
                    LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                    deleteFailCount++;
                }
            }

            if (deleteSuccessCount >= 1) {
                // 删除当前项目
                int deleteCount = projectMapper.deleteById(id);
                if (deleteCount == 1) {
                    LOGGER.info("project表结删除改成功，本地文件夹删除成功");
                    value = true;
                } else {
                    LOGGER.error("project表结构删除失败,本地文件夹删除失败");
                    value = false;
                }
            } else {
                value = false;
            }


        }
        return value;
    }

    @Override
    public List<Project> listProject() {
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.orderByDesc(Project::getProjectTime);
        List<Project> projects = projectMapper.selectList(queryWrapper);
        for (Project project : projects){
            StorageSource storageSource =   getStorageConfig(project.getLocalStorageId());
            project.setKey(storageSource.getKey());
            project.setType(String.valueOf(storageSource.getType()));
        }
        return projects;
    }

    // 辅助方法：获取存储配置
    private StorageSource getStorageConfig(Integer id) {
        return storageSourceMapper.selectOne(new LambdaQueryWrapper<StorageSource>().eq(StorageSource::getId, id)
        );
    }
}
