package com.yfd.platform.modules.storage.convert.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yfd.platform.modules.config.model.request.SaveStorageSourceRequest;
import com.yfd.platform.modules.experimentalData.domain.TsTask;
import com.yfd.platform.modules.experimentalData.mapper.TsTaskMapper;
import com.yfd.platform.modules.readme.model.entity.ReadmeConfig;
import com.yfd.platform.modules.specialDocument.domain.Project;
import com.yfd.platform.modules.specialDocument.mapper.ProjectMapper;
import com.yfd.platform.modules.storage.convert.StorageSourceConvert;
import com.yfd.platform.modules.storage.mapper.StorageSourceConfigMapper;
import com.yfd.platform.modules.storage.model.dto.StorageSourceAllParamDTO;
import com.yfd.platform.modules.storage.model.dto.StorageSourceDTO;
import com.yfd.platform.modules.storage.model.entity.StorageSource;
import com.yfd.platform.modules.storage.model.entity.StorageSourceConfig;
import com.yfd.platform.modules.storage.model.result.StorageSourceAdminResult;
import com.yfd.platform.modules.storage.model.result.StorageSourceConfigResult;
import com.yfd.platform.modules.storage.model.result.StorageSourceResult;
import com.yfd.platform.utils.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.nio.file.FileStore;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Date: 2025/1/10 12:44
 * @Description:
 */
@Service
public class StorageSourceConvertImpl implements StorageSourceConvert {

    @Resource
    private StorageSourceConfigMapper storageSourceConfigMapper;

    //专项项目表Mapper
    @Resource
    private ProjectMapper projectMapper;

    //试验任务Mapper
    @Resource
    private TsTaskMapper tsTaskMapper;


    @Override
    public List<StorageSourceResult> entityToResultList(List<StorageSource> list) {
        if ( list == null ) {
            return null;
        }

        List<StorageSourceResult> list1 = new ArrayList<StorageSourceResult>( list.size() );
        for ( StorageSource storageSource : list ) {
            list1.add( storageSourceToStorageSourceResult( storageSource ) );
        }

        return list1;
    }

    @Override
    public StorageSourceConfigResult entityToConfigResult(StorageSource storageSource, ReadmeConfig readmeConfig) {
        if ( storageSource == null && readmeConfig == null ) {
            return null;
        }

        StorageSourceConfigResult storageSourceConfigResult = new StorageSourceConfigResult();

        if ( storageSource != null ) {
            storageSourceConfigResult.setEnableFileOperator( storageSource.getEnableFileOperator() );
            storageSourceConfigResult.setDefaultSwitchToImgMode( storageSource.getDefaultSwitchToImgMode() );
        }
        if ( readmeConfig != null ) {
            storageSourceConfigResult.setReadmeDisplayMode( readmeConfig.getDisplayMode() );
            storageSourceConfigResult.setReadmeText( readmeConfig.getReadmeText() );
        }

        return storageSourceConfigResult;
    }

    @Override
    public List<StorageSourceAdminResult> entityToAdminResultList(List<StorageSource> list) {
        if (list == null) {
            return Collections.emptyList();
        }

        return list.stream()
                .map(this::storageSourceToStorageSourceAdminResult)
                .collect(Collectors.toList());
    }

    @Override
    public Page<StorageSourceAdminResult> entityToAdminResultPage(Page<StorageSource> sourcePage) {
        if (sourcePage == null) {
            return new Page<>();
        }
        List<StorageSource> recordData = sourcePage.getRecords();
        for (StorageSource storageSource : recordData) {
            StringBuilder result = new StringBuilder(); // 用于拼接结果
            String type = storageSource.getType().getKey();
            if ("local".equals(type)) {
                //获取关联的存储内容  先获取专项文档
                LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(Project::getLocalStorageId, storageSource.getId());
                List<Project> projects = projectMapper.selectList(queryWrapper);
                if (projects.size()>0){
                    for (Project project : projects) {
                        if (result.length() > 0) {
                            result.append(", "); // 如果结果不为空，加上逗号分隔
                        }
                        result.append(project.getProjectName()); // 添加项目名称
                    }

                }

                LambdaQueryWrapper<TsTask> queryWrapper1 = new LambdaQueryWrapper<>();
                queryWrapper1.eq(TsTask::getLocalStorageId, storageSource.getId());
                List<TsTask> tsTasks = tsTaskMapper.selectList(queryWrapper1);
                if (tsTasks.size()>0){
                    for (TsTask task : tsTasks) {
                        if (result.length() > 0) {
                            result.append(", "); // 如果结果不为空，加上逗号分隔
                        }
                        result.append(task.getTaskName()); // 添加任务名称
                    }
                }

                storageSource.setStoreContent(String.valueOf(result));
                //存储路径
                LambdaQueryWrapper<StorageSourceConfig> queryWrapperData = new LambdaQueryWrapper<>();
                queryWrapperData.eq(StorageSourceConfig::getStorageId, storageSource.getId());
                queryWrapperData.eq(StorageSourceConfig::getName, "filePath");
                String value = storageSourceConfigMapper.selectOne(queryWrapperData).getValue();
                storageSource.setValueData(value);

                if(StringUtils.isNotBlank(value)){
                    //空间使用率
                    storageSource.setSpaceOccupancyRatio(calculateLocalStorageUsage(value));
                }

            }else {
                LambdaQueryWrapper<TsTask> queryWrapper1 = new LambdaQueryWrapper<>();
                queryWrapper1.eq(TsTask::getBackupStorageId, storageSource.getId());
                List<TsTask> tsTasks = tsTaskMapper.selectList(queryWrapper1);
                if (tsTasks.size()>0){
                    for (TsTask task : tsTasks) {
                        if (result.length() > 0) {
                            result.append(", "); // 如果结果不为空，加上逗号分隔
                        }
                        result.append(task.getTaskName()); // 添加任务名称
                    }
                }
                storageSource.setStoreContent(String.valueOf(result));
                //存储路径
                LambdaQueryWrapper<StorageSourceConfig> queryWrapperData = new LambdaQueryWrapper<>();
                queryWrapperData.eq(StorageSourceConfig::getStorageId, storageSource.getId());
                queryWrapperData.eq(StorageSourceConfig::getName, "filePath");
                String value = storageSourceConfigMapper.selectOne(queryWrapperData).getValue();
                storageSource.setValueData(value);
                if(StringUtils.isNotBlank(value)){
                    //空间使用率
                    storageSource.setSpaceOccupancyRatio(calculateLocalStorageUsage(value));
                }


            }
        }


        // 转换记录列表
        List<StorageSourceAdminResult> records = entityToAdminResultList(sourcePage.getRecords());

        // 创建目标分页对象并复制所有分页属性
        Page<StorageSourceAdminResult> resultPage = new Page<>();

        // 复制分页属性
        resultPage.setRecords(records);
        resultPage.setTotal(sourcePage.getTotal());
        resultPage.setSize(sourcePage.getSize());
        resultPage.setCurrent(sourcePage.getCurrent());
        resultPage.setOrders(sourcePage.getOrders());
        resultPage.setOptimizeCountSql(sourcePage.optimizeCountSql());
        resultPage.setSearchCount(sourcePage.isSearchCount());
        resultPage.setCountId(sourcePage.getCountId());
        resultPage.setMaxLimit(sourcePage.getMaxLimit());



        return resultPage;
    }

    // 计算本地路径的空间使用率
    public  static String  calculateLocalStorageUsage(String path) {

        try {
            Path target = Paths.get(path);

            // 处理符号链接（获取实际挂载点）
            if (Files.isSymbolicLink(target)) {
                target = Files.readSymbolicLink(target);
            }

            // 获取文件存储信息
            FileStore store = Files.getFileStore(target);

            long totalSpace = store.getTotalSpace();
            long usableSpace = store.getUsableSpace();

            if (totalSpace <= 0) {
                return "0%"; // 特殊文件系统处理
            }

            double usagePercentage = (double) (totalSpace - usableSpace) / totalSpace * 100;
            DecimalFormat df = new DecimalFormat("#.##");
            return df.format(usagePercentage) + "%";

        } catch (Exception e) {
            throw new IllegalArgumentException("无法获取磁盘空间: " + e.getMessage());
        }
//        File file = new File(path);
//        if (!file.exists()) {
//            throw new IllegalArgumentException("路径不存在: " + path);
//        }
//
//        // 获取磁盘空间信息
//        long totalSpace = file.getTotalSpace();
//        long freeSpace = file.getFreeSpace();
//        // 计算使用率
//        long usedSpace = totalSpace - freeSpace;
//        double usagePercentage = (double) usedSpace / totalSpace * 100;
//
//        // 格式化输出
//        DecimalFormat df = new DecimalFormat("#.##");
//
//        return df.format(usagePercentage) + "%";

    }





    @Override
    public StorageSourceDTO entityToDTO(StorageSource storageSource, StorageSourceAllParamDTO storageSourceAllParam) {
        if ( storageSource == null && storageSourceAllParam == null ) {
            return null;
        }

        StorageSourceDTO storageSourceDTO = new StorageSourceDTO();

        if ( storageSource != null ) {
            storageSourceDTO.setId( storageSource.getId() );
            storageSourceDTO.setName( storageSource.getName() );
            storageSourceDTO.setKey( storageSource.getKey() );
            storageSourceDTO.setRemark( storageSource.getRemark() );
            storageSourceDTO.setType( storageSource.getType() );
            if ( storageSource.getEnable() != null ) {
                storageSourceDTO.setEnable( storageSource.getEnable() );
            }
            storageSourceDTO.setEnableFileOperator( storageSource.getEnableFileOperator() );
            storageSourceDTO.setEnableFileAnnoOperator( storageSource.getEnableFileAnnoOperator() );
            if ( storageSource.getEnableCache() != null ) {
                storageSourceDTO.setEnableCache( storageSource.getEnableCache() );
            }
            if ( storageSource.getAutoRefreshCache() != null ) {
                storageSourceDTO.setAutoRefreshCache( storageSource.getAutoRefreshCache() );
            }
            if ( storageSource.getSearchEnable() != null ) {
                storageSourceDTO.setSearchEnable( storageSource.getSearchEnable() );
            }
            if ( storageSource.getSearchIgnoreCase() != null ) {
                storageSourceDTO.setSearchIgnoreCase( storageSource.getSearchIgnoreCase() );
            }
            storageSourceDTO.setOrderNum( storageSource.getOrderNum() );
            if ( storageSource.getDefaultSwitchToImgMode() != null ) {
                storageSourceDTO.setDefaultSwitchToImgMode( storageSource.getDefaultSwitchToImgMode() );
            }
            storageSourceDTO.setCompatibilityReadme( storageSource.getCompatibilityReadme() );
        }
        storageSourceDTO.setStorageSourceAllParam( storageSourceAllParam );

        return storageSourceDTO;
    }

    @Override
    public StorageSource saveRequestToEntity(SaveStorageSourceRequest saveStorageSourceRequest) {
        if ( saveStorageSourceRequest == null ) {
            return null;
        }

        StorageSource storageSource = new StorageSource();

        storageSource.setId( saveStorageSourceRequest.getId() );
        storageSource.setEnable( saveStorageSourceRequest.getEnable() );
        storageSource.setEnableFileOperator( saveStorageSourceRequest.getEnableFileOperator() );
        storageSource.setEnableFileAnnoOperator( saveStorageSourceRequest.getEnableFileAnnoOperator() );
        storageSource.setEnableCache( saveStorageSourceRequest.isEnableCache() );
        storageSource.setName( saveStorageSourceRequest.getName() );
        storageSource.setKey( saveStorageSourceRequest.getKey() );
        storageSource.setRemark( saveStorageSourceRequest.getRemark() );
        storageSource.setAutoRefreshCache( saveStorageSourceRequest.isAutoRefreshCache() );
        storageSource.setType( saveStorageSourceRequest.getType() );
        storageSource.setSearchEnable( saveStorageSourceRequest.isSearchEnable() );
        storageSource.setSearchIgnoreCase( saveStorageSourceRequest.isSearchIgnoreCase() );
        storageSource.setSearchMode( saveStorageSourceRequest.getSearchMode() );
        storageSource.setOrderNum( saveStorageSourceRequest.getOrderNum() );
        storageSource.setDefaultSwitchToImgMode( saveStorageSourceRequest.isDefaultSwitchToImgMode() );
        storageSource.setCompatibilityReadme( saveStorageSourceRequest.isCompatibilityReadme() );

        return storageSource;
    }

    protected StorageSourceResult storageSourceToStorageSourceResult(StorageSource storageSource) {
        if ( storageSource == null ) {
            return null;
        }

        StorageSourceResult storageSourceResult = new StorageSourceResult();

        storageSourceResult.setName( storageSource.getName() );
        storageSourceResult.setKey( storageSource.getKey() );
        storageSourceResult.setType( storageSource.getType() );
        storageSourceResult.setSearchEnable( storageSource.getSearchEnable() );
        storageSourceResult.setDefaultSwitchToImgMode( storageSource.getDefaultSwitchToImgMode() );

        return storageSourceResult;
    }

    protected StorageSourceAdminResult storageSourceToStorageSourceAdminResult(StorageSource storageSource) {
        if ( storageSource == null ) {
            return null;
        }

        StorageSourceAdminResult storageSourceAdminResult = new StorageSourceAdminResult();

        storageSourceAdminResult.setId( storageSource.getId() );
        storageSourceAdminResult.setEnable( storageSource.getEnable() );
        storageSourceAdminResult.setEnableFileOperator( storageSource.getEnableFileOperator() );
        storageSourceAdminResult.setEnableFileAnnoOperator( storageSource.getEnableFileAnnoOperator() );
        storageSourceAdminResult.setEnableCache( storageSource.getEnableCache() );
        storageSourceAdminResult.setName( storageSource.getName() );
        storageSourceAdminResult.setKey( storageSource.getKey() );
        storageSourceAdminResult.setRemark( storageSource.getRemark() );
        storageSourceAdminResult.setAutoRefreshCache( storageSource.getAutoRefreshCache() );
        storageSourceAdminResult.setType( storageSource.getType() );
        storageSourceAdminResult.setSearchEnable( storageSource.getSearchEnable() );
        storageSourceAdminResult.setSearchIgnoreCase( storageSource.getSearchIgnoreCase() );
        storageSourceAdminResult.setSearchMode( storageSource.getSearchMode() );
        storageSourceAdminResult.setOrderNum( storageSource.getOrderNum() );
        storageSourceAdminResult.setDefaultSwitchToImgMode( storageSource.getDefaultSwitchToImgMode() );
        storageSourceAdminResult.setCompatibilityReadme( storageSource.getCompatibilityReadme() );
        storageSourceAdminResult.setStoreContent( storageSource.getStoreContent());
        storageSourceAdminResult.setSpaceOccupancyRatio( storageSource.getSpaceOccupancyRatio());
        storageSourceAdminResult.setValueData( storageSource.getValueData());
        return storageSourceAdminResult;
    }
}
